package org.apidesign.apifest08.currency;

/**
 * Interface declaring method for computing conversion.
 * 
 * Because of a vague definition of currency amount's type,
 * the interface has a generic type.
 * 
 * @author ked
 * @see http://wiki.apidesign.org/wiki/APIDesignPatterns:ResponseReply
 */
interface Computer<AmountType> {

    ComputerResponse<AmountType> compute(ComputerRequest<AmountType> request);

    /**
     * 
     * @param <AmountType>
     */
    final class ComputerRequest<AmountType> {

        private AmountType input;
        private AmountType inputCurrencyRatio;
        private AmountType outputCurrencyRatio;

        AmountType getInput() {
            return input;
        }

        void setInput(AmountType input) {
            this.input = input;
        }

        AmountType getInputCurrencyRatio() {
            return inputCurrencyRatio;
        }

        void setInputCurrencyRatio(AmountType inputCurrencyRatio) {
            this.inputCurrencyRatio = inputCurrencyRatio;
        }

        AmountType getOutputCurrencyRatio() {
            return outputCurrencyRatio;
        }

        void setOutputCurrencyRatio(AmountType outputCurrencyRatio) {
            this.outputCurrencyRatio = outputCurrencyRatio;
        }
    }

    final class ComputerResponse<AmountType> {

        private AmountType result;

        AmountType getResult() {
            return result;
        }

        void setResult(AmountType result) {
            this.result = result;
        }
    }
    
    
}