package org.apidesign.apifest08.test;

import java.math.BigDecimal;
import org.apidesign.apifest08.currency.ConvertorCurrency;
import org.apidesign.apifest08.currency.ExchangeRate;
import org.apidesign.apifest08.currency.IExchangeRateEngine;

/** Exchange rate engine which is periodicaly changing echange rate.
 *  See <a href="http://wiki.apidesign.org/wiki/APIFest08:Task3">http://wiki.apidesign.org/wiki/APIFest08:Task3</a> for further description.
 *
 * @author arnostvalicek
 */
public class BouncingExchangeRateEngine implements IExchangeRateEngine {
    
    ConvertorCurrency fromCurrency;
    BigDecimal fromValue;
    ConvertorCurrency toCurrency;
    BigDecimal toValueStart;
    BigDecimal toValueStop;
    BigDecimal step;
    
    BigDecimal toValue;

    
    
    
    private BouncingExchangeRateEngine(ConvertorCurrency fromCurrency, BigDecimal fromValue, ConvertorCurrency toCurrency, BigDecimal toValueStart, BigDecimal toValueStop, BigDecimal step) {
        this.fromCurrency = fromCurrency;
        this.fromValue = fromValue;
        this.toCurrency = toCurrency;
        this.toValueStart = toValueStart;
        this.toValueStop = toValueStop;
        this.step = step;
        this.toValue = toValueStart;
    }


    public static IExchangeRateEngine create(ConvertorCurrency fromCurrency, BigDecimal fromValue, ConvertorCurrency toCurrency, BigDecimal toValueStart, BigDecimal toValueStop, BigDecimal step) {
        return new BouncingExchangeRateEngine(fromCurrency, fromValue, toCurrency, toValueStart, toValueStop, step);
    }

    public ExchangeRate getExchangeRate(ConvertorCurrency fromCurrency, ConvertorCurrency toCurrency) {
        if (!fromCurrency.equals(this.fromCurrency)) {
            return null;
        }
        if (!toCurrency.equals(this.toCurrency)) {
            return null;
        }
        
        ExchangeRate result = new ExchangeRate(fromValue, toValue);
        
        toValue = toValue.add(step);
        
//        if (step.signum()==1 && toValueStop.compareTo(toValue)==0) {
//            System.out.println("A");
//            toValue=toValueStop;
//            step = step.negate();
//            BigDecimal x = toValueStart;
//            toValueStart=toValueStop;
//            toValueStop=x;
//        } else if (step.signum()==-1 && toValueStop.compareTo(toValue)==0) {
//            System.out.println("B");
//            toValue=toValueStop;
//            step = step.negate();
//            BigDecimal x = toValueStart;
//            toValueStart=toValueStop;
//            toValueStop=x;
//        }
        if (toValueStop.compareTo(toValue)==0) {
            toValue=toValueStop;
            step = step.negate();
            BigDecimal x = toValueStart;
            toValueStart=toValueStop;
            toValueStop=x;
        }        
        return result;
    }

}
