package org.apidesign.apifest08.currency;

import java.math.BigDecimal;
import java.util.Currency;
import java.util.Date;


final class DatedConvertorImpl
    extends    ConvertorImpl
    implements DatedConvertor
{
    private final DateRange range;
    
    DatedConvertorImpl(final DateRange rng,
                       final ExchangeRate r)
    {
        super(r);
        
        if(rng == null)
        {
            throw new IllegalArgumentException("rng cannot be null");
        }

        range = rng;
    }
    
    public DateRange getDateRange()
    {
        return (range);
    }

    @Override
    public BigDecimal convert(final Currency   from,
                              final Currency   to,
                              final BigDecimal amount)
        throws InvalidConversionException
    {
        final BigDecimal total;
        
        total = convert(from, to, amount, new Date(System.currentTimeMillis()));
        
        return (total);
    }

    public BigDecimal convert(final Currency   from,
                              final Currency   to,
                              final BigDecimal amount,
                              final Date       date)
        throws InvalidConversionException
    {
        final BigDecimal total;

        if(range.isInRange(date))
        {
            total = super.convert(from, to, amount);
        }
        else
        {
            throw new InvalidConversionException("cannot convert for date", from);
        }

        return (total);
    }

    public boolean canConvert(final Currency from,
                              final Currency to,
                              final Date     date)
    {
        final boolean retVal;

        if(canConvert(from, to))
        {
            retVal = range.isInRange(date);
        }
        else
        {
            retVal = false;
        }

        return (retVal);
    }
}
