package apifest;

import java.math.BigDecimal;
import junit.framework.TestCase;
import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.ConvertorCurrency;
import org.apidesign.apifest08.currency.ExchangeRateProvider;


/** Write a test that works with version from task A and fails with version B.
 */
public class CurrencyTest extends TestCase {
    public CurrencyTest(String n) {
        super(n);
    }
    
    /** Fails because runtime incompatibility. */
    public void testCompatibility() throws Exception {
        ConvertorCurrency cur1 = ConvertorCurrency.getInstance("CZK");
        ConvertorCurrency cur2 = ConvertorCurrency.getInstance("CZK");
        
        assertEquals(false, cur1.equals(cur2));
    }

    /** Fails because source incompatibility. */
    public void testCompatibility2() throws Exception {
        MyOwnExchangeRateProvider provider = new MyOwnExchangeRateProvider(
                new BigDecimal("1"), ConvertorCurrency.getInstance("USD"),
                new BigDecimal("16"), ConvertorCurrency.getInstance("CZK"));
        Convertor c = Convertor.createConvertor(provider);
        
        boolean result = provider.addFixedCurencyRate(
                ConvertorCurrency.getInstance("CZK"), new BigDecimal("80"),
                ConvertorCurrency.getInstance("SKK"), new BigDecimal("100"));
        assertEquals(true, result);
    }
    
    class MyOwnExchangeRateProvider extends ExchangeRateProvider {

        public MyOwnExchangeRateProvider(BigDecimal fromValue, ConvertorCurrency fromCurrency, BigDecimal toValue, ConvertorCurrency toCurrency) {
            super(fromValue, fromCurrency, toValue, toCurrency);
        }
    
        public boolean addFixedCurencyRate(ConvertorCurrency fromCurrency, BigDecimal fromValue, ConvertorCurrency toCurrency, BigDecimal toValue) {
            return true; // great method - do nothing and is always satisfied ;-)
        }

    }
}
